@@ -4,11 +4,14 @@
 
 from __future__ import annotations  # pylint: disable=unused-variable
 
+import json
 import logging
 from pathlib import Path
-from typing import Dict, List, NewType
+import time
+from typing import Dict, List, NewType, Iterable
 import requests
 
+from aistore.sdk.ais_source import AISSource
 from aistore.sdk.etl_const import DEFAULT_ETL_TIMEOUT
 from aistore.sdk.object_iterator import ObjectIterator
 from aistore.sdk.const import (
@@ -19,22 +22,35 @@ from aistore.sdk.const import (
     ACT_EVICT_REMOTE_BCK,
     ACT_LIST,
     ACT_MOVE_BCK,
+    ACT_SUMMARY_BCK,
+    HEADER_ACCEPT,
+    HEADER_BUCKET_PROPS,
+    HEADER_BUCKET_SUMM,
+    HEADER_XACTION_ID,
     HTTP_METHOD_DELETE,
     HTTP_METHOD_GET,
     HTTP_METHOD_HEAD,
     HTTP_METHOD_POST,
+    MSGPACK_CONTENT_TYPE,
     PROVIDER_AIS,
     QPARAM_BCK_TO,
+    QPARAM_BSUMM_REMOTE,
+    QPARAM_FLT_PRESENCE,
     QPARAM_KEEP_REMOTE,
     QPARAM_NAMESPACE,
     QPARAM_PROVIDER,
+    QPARAM_UUID,
     URL_PATH_BUCKETS,
+    STATUS_ACCEPTED,
+    STATUS_OK,
+    STATUS_PARTIAL_CONTENT,
 )
 
 from aistore.sdk.errors import (
     InvalidBckProvider,
     ErrBckAlreadyExists,
     ErrBckNotFound,
+    UnexpectedHTTPStatusCode,
 )
 from aistore.sdk.multiobj import ObjectGroup, ObjectRange
 from aistore.sdk.request_client import RequestClient
@@ -44,6 +60,7 @@ from aistore.sdk.types import (
     BucketEntry,
     BucketList,
     BucketModel,
+    BsummCtrlMsg,
     Namespace,
     CopyBckMsg,
     TransformBckMsg,
@@ -57,7 +74,7 @@ Header = NewType("Header", requests.structures.CaseInsensitiveDict)
 
 
 # pylint: disable=unused-variable,too-many-public-methods
-class Bucket:
+class Bucket(AISSource):
     """
     A class representing a bucket that contains user data.
 
@@ -108,6 +125,19 @@ class Bucket:
         """The namespace for this bucket."""
         return self._namespace
 
+    def list_urls(self, prefix: str = "", etl_name: str = None) -> Iterable[str]:
+        """
+            Get an iterator of full URLs to every object in this bucket matching the prefix
+        Args:
+            prefix (str, optional): Limit objects selected by a given string prefix
+            etl_name (str, optional): ETL to include in URLs
+
+        Returns:
+            Iterator of all object URLs matching the prefix
+        """
+        for entry in self.list_objects_iter(prefix=prefix, props="name"):
+            yield self.object(entry.name).get_url(etl_name=etl_name)
+
     def create(self, exist_ok=False):
         """
         Creates a bucket in AIStore cluster.
@@ -233,6 +263,161 @@ class Bucket:
             params=self.qparam,
         ).headers
 
+    # pylint: disable=too-many-arguments
+    def summary(
+        self,
+        uuid: str = "",
+        prefix: str = "",
+        cached: bool = True,
+        present: bool = True,
+    ):
+        """
+        Returns bucket summary (starts xaction job and polls for results).
+
+        Args:
+            uuid (str): Identifier for the bucket summary. Defaults to an empty string.
+            prefix (str): Prefix for objects to be included in the bucket summary.
+                          Defaults to an empty string (all objects).
+            cached (bool): If True, summary entails cached entities. Defaults to True.
+            present (bool): If True, summary entails present entities. Defaults to True.
+
+        Raises:
+            requests.ConnectionError: Connection error
+            requests.ConnectionTimeout: Timed out connecting to AIStore
+            requests.exceptions.HTTPError: Service unavailable
+            requests.RequestException: "There was an ambiguous exception that occurred while handling..."
+            requests.ReadTimeout: Timed out receiving response from AIStore
+            aistore.sdk.errors.AISError: All other types of errors with AIStore
+        """
+        bsumm_ctrl_msg = BsummCtrlMsg(
+            uuid=uuid, prefix=prefix, cached=cached, present=present
+        )
+
+        # Start the job and get the job ID
+        resp = self.make_request(
+            HTTP_METHOD_GET,
+            ACT_SUMMARY_BCK,
+            params=self.qparam,
+            value=bsumm_ctrl_msg.dict(),
+        )
+
+        # Initial response status code should be 202
+        if resp.status_code == STATUS_OK:
+            raise UnexpectedHTTPStatusCode([STATUS_ACCEPTED], resp.status_code)
+
+        job_id = resp.text.strip('"')
+
+        # Update the uuid in the control message
+        bsumm_ctrl_msg.uuid = job_id
+
+        # Sleep and request frequency in sec (starts at 200 ms)
+        sleep_time = 0.2
+
+        # Poll async task for http.StatusOK completion
+        while True:
+            resp = self.make_request(
+                HTTP_METHOD_GET,
+                ACT_SUMMARY_BCK,
+                params=self.qparam,
+                value=bsumm_ctrl_msg.dict(),
+            )
+
+            # If task completed successfully, break the loop
+            if resp.status_code == STATUS_OK:
+                break
+            # If task is still running, wait for some time and try again
+            if resp.status_code == STATUS_ACCEPTED:
+                time.sleep(sleep_time)
+                sleep_time = min(
+                    10, sleep_time * 1.5
+                )  # Increase sleep_time by 50%, but don't exceed 10 seconds
+            # Otherwise, if status code received is neither STATUS_OK or STATUS_ACCEPTED, raise an exception
+            else:
+                raise UnexpectedHTTPStatusCode(
+                    [STATUS_OK, STATUS_ACCEPTED], resp.status_code
+                )
+
+        return json.loads(resp.content.decode("utf-8"))[0]
+
+    def info(self, flt_presence: int = 0, bsumm_remote: bool = True):
+        """
+        Returns bucket summary and information/properties.
+
+        Args:
+            flt_presence (int): Describes the presence of buckets and objects with respect to their existence
+                                or non-existence in the AIS cluster. Defaults to 0.
+
+                                Expected values are:
+                                0 - (object | bucket) exists inside and/or outside cluster
+                                1 - same as 0 but no need to return summary
+                                2 - bucket: is present | object: present and properly located
+                                3 - same as 2 but no need to return summary
+                                4 - objects: present anywhere/anyhow _in_ the cluster as: replica, ec-slices, misplaced
+                                5 - not present - exists _outside_ cluster
+            bsumm_remote (bool): If True, returned bucket info will include remote objects as well
+
+        Raises:
+            requests.ConnectionError: Connection error
+            requests.ConnectionTimeout: Timed out connecting to AIStore
+            requests.exceptions.HTTPError: Service unavailable
+            requests.RequestException: "There was an ambiguous exception that occurred while handling..."
+            requests.ReadTimeout: Timed out receiving response from AIStore
+            ValueError: `flt_presence` is not one of the expected values
+            aistore.sdk.errors.AISError: All other types of errors with AIStore
+        """
+        if flt_presence not in range(6):
+            raise ValueError("`flt_presence` must be one of 0, 1, 2, 3, 4, or 5.")
+
+        params = self.qparam.copy()
+        params.update({QPARAM_FLT_PRESENCE: flt_presence})
+        params[QPARAM_BSUMM_REMOTE] = bsumm_remote
+
+        response = self.client.request(
+            HTTP_METHOD_HEAD,
+            path=f"{URL_PATH_BUCKETS}/{self.name}",
+            params=params,
+        )
+
+        bucket_props = response.headers.get(HEADER_BUCKET_PROPS, "{}")
+        uuid = response.headers.get(HEADER_XACTION_ID, "").strip('"')
+        params[QPARAM_UUID] = uuid
+
+        # Initial response status code should be 202
+        if response.status_code != int(STATUS_ACCEPTED):
+            raise UnexpectedHTTPStatusCode([STATUS_ACCEPTED], response.status_code)
+
+        # Sleep and request frequency in sec (starts at 2 s)
+        sleep_time = 2
+        time.sleep(sleep_time)
+        i = 0
+
+        # Poll async task for http.StatusOK completion
+        while True:
+            response = self.client.request(
+                HTTP_METHOD_HEAD,
+                path=f"{URL_PATH_BUCKETS}/{self.name}",
+                params=params,
+            )
+
+            bucket_summ = response.headers.get(HEADER_BUCKET_SUMM, "")
+
+            if bucket_summ != "":
+                result = json.loads(bucket_summ)
+
+            # If task completed successfully, break the loop
+            if response.status_code == STATUS_OK:
+                break
+
+            time.sleep(sleep_time)
+            i += 1
+
+            if i == 8 and response.status_code != STATUS_PARTIAL_CONTENT:
+                sleep_time *= 2
+            elif i == 16 and response.status_code != STATUS_PARTIAL_CONTENT:
+                sleep_time *= 2
+
+        return bucket_props, result
+
     # pylint: disable=too-many-arguments
     def copy(
         self,
@@ -241,6 +426,8 @@ class Bucket:
         prepend: str = "",
         dry_run: bool = False,
         force: bool = False,
+        latest: bool = False,
+        sync: bool = False,
     ) -> str:
         """
         Returns job ID that can be used later to check the status of the asynchronous operation.
@@ -252,6 +439,8 @@ class Bucket:
             dry_run (bool, optional): Determines if the copy should actually
                 happen or not
             force (bool, optional): Override existing destination bucket
+            latest (bool, optional): GET the latest object version from the associated remote bucket
+            sync (bool, optional): synchronize destination bucket with its remote (e.g., Cloud or remote AIS) source
 
         Returns:
             Job ID (as str) that can be used to check the status of the operation
@@ -265,7 +454,12 @@ class Bucket:
             requests.ReadTimeout: Timed out receiving response from AIStore
         """
         value = CopyBckMsg(
-            prefix=prefix_filter, prepend=prepend, dry_run=dry_run, force=force
+            prefix=prefix_filter,
+            prepend=prepend,
+            dry_run=dry_run,
+            force=force,
+            latest=latest,
+            sync=sync,
         ).as_dict()
         params = self.qparam.copy()
         params[QPARAM_BCK_TO] = to_bck.get_path()
@@ -331,10 +525,12 @@ class Bucket:
         bucket_list = self.client.request_deserialize(
             HTTP_METHOD_GET,
             path=f"{URL_PATH_BUCKETS}/{ self.name }",
+            headers={HEADER_ACCEPT: MSGPACK_CONTENT_TYPE},
             res_model=BucketList,
             json=action,
             params=self.qparam,
         )
+
         for entry in bucket_list.entries:
             entry.object = self.object(entry.name)
         return bucket_list
@@ -440,8 +636,8 @@ class Bucket:
                 target=target,
             )
             if obj_list:
-                obj_list = obj_list + resp.get_entries()
-            obj_list = obj_list or resp.get_entries()
+                obj_list = obj_list + resp.entries
+            obj_list = obj_list or resp.entries
             if resp.continuation_token == "":
                 break
             continuation_token = resp.continuation_token
@@ -459,6 +655,8 @@ class Bucket:
         ext: Dict[str, str] = None,
         force: bool = False,
         dry_run: bool = False,
+        latest: bool = False,
+        sync: bool = False,
     ) -> str:
         """
         Visits all selected objects in the source bucket and for each object, puts the transformed
@@ -474,6 +672,8 @@ class Bucket:
                 (i.e. {"jpg": "txt"})
             dry_run (bool, optional): determines if the copy should actually happen or not
             force (bool, optional): override existing destination bucket
+            latest (bool, optional): GET the latest object version from the associated remote bucket
+            sync (bool, optional): synchronize destination bucket with its remote (e.g., Cloud or remote AIS) source
 
         Returns:
             Job ID (as str) that can be used to check the status of the operation
@@ -482,7 +682,12 @@ class Bucket:
             ext=ext,
             transform_msg=TransformBckMsg(etl_name=etl_name, timeout=timeout),
             copy_msg=CopyBckMsg(
-                prefix=prefix_filter, prepend=prepend, force=force, dry_run=dry_run
+                prefix=prefix_filter,
+                prepend=prepend,
+                force=force,
+                dry_run=dry_run,
+                latest=latest,
+                sync=sync,
             ),
         ).as_dict()
 
