@@ -2,9 +2,11 @@
 # Copyright (c) 2022-2023, NVIDIA CORPORATION. All rights reserved.
 #
 from io import BufferedWriter
-from typing import NewType
+from typing import NewType, Iterable
+
 import requests
 
+from aistore.sdk.ais_source import AISSource
 from aistore.sdk.const import (
     DEFAULT_CHUNK_SIZE,
     HTTP_METHOD_DELETE,
@@ -13,6 +15,7 @@ from aistore.sdk.const import (
     HTTP_METHOD_PUT,
     QPARAM_ARCHPATH,
     QPARAM_ETL_NAME,
+    QPARAM_LATEST,
     ACT_PROMOTE,
     HTTP_METHOD_POST,
     URL_PATH_OBJECTS,
@@ -26,7 +29,7 @@ Header = NewType("Header", requests.structures.CaseInsensitiveDict)
 
 
 # pylint: disable=consider-using-with,unused-variable
-class Object:
+class Object(AISSource):
     """
     A class representing an object of a bucket bound to a client.
 
@@ -42,6 +45,7 @@ class Object:
         self._bck_name = bucket.name
         self._qparams = bucket.qparam
         self._name = name
+        self._object_path = f"{URL_PATH_OBJECTS}/{ self._bck_name}/{ self.name }"
 
     @property
     def bucket(self):
@@ -53,6 +57,9 @@ class Object:
         """Name of this object"""
         return self._name
 
+    def list_urls(self, prefix: str = "", etl_name: str = None) -> Iterable[str]:
+        yield self.get_url(etl_name=etl_name)
+
     def head(self) -> Header:
         """
         Requests object properties.
@@ -69,16 +76,18 @@ class Object:
         """
         return self._client.request(
             HTTP_METHOD_HEAD,
-            path=f"{URL_PATH_OBJECTS}/{ self._bck_name}/{ self.name }",
+            path=self._object_path,
             params=self._qparams,
         ).headers
 
+    # pylint: disable=too-many-arguments
     def get(
         self,
         archpath: str = "",
         chunk_size: int = DEFAULT_CHUNK_SIZE,
         etl_name: str = None,
         writer: BufferedWriter = None,
+        latest: bool = False,
     ) -> ObjectReader:
         """
         Reads an object
@@ -88,8 +97,9 @@ class Object:
                 from the archive
             chunk_size (int, optional): chunk_size to use while reading from stream
             etl_name (str, optional): Transforms an object based on ETL with etl_name
-            writer (BufferedWriter, optional): User-provided writer for writing content output.
+            writer (BufferedWriter, optional): User-provided writer for writing content output
                 User is responsible for closing the writer
+            latest (bool, optional): GET the latest object version from the associated remote bucket
 
         Returns:
             The stream of bytes to read an object or a file inside an archive.
@@ -104,9 +114,11 @@ class Object:
         params[QPARAM_ARCHPATH] = archpath
         if etl_name:
             params[QPARAM_ETL_NAME] = etl_name
+        if latest:
+            params[QPARAM_LATEST] = "true"
         resp = self._client.request(
             HTTP_METHOD_GET,
-            path=f"{URL_PATH_OBJECTS}/{ self._bck_name }/{ self.name }",
+            path=self._object_path,
             params=params,
             stream=True,
         )
@@ -119,6 +131,26 @@ class Object:
             writer.writelines(obj_reader)
         return obj_reader
 
+    def get_url(self, archpath: str = "", etl_name: str = None):
+        """
+        Get the full url to the object including base url and any query parameters
+
+        Args:
+            archpath (str, optional): If the object is an archive, use `archpath` to extract a single file
+                from the archive
+            etl_name (str, optional): Transforms an object based on ETL with etl_name
+
+        Returns:
+            Full URL to get object
+
+        """
+        params = self._qparams.copy()
+        if archpath:
+            params[QPARAM_ARCHPATH] = archpath
+        if etl_name:
+            params[QPARAM_ETL_NAME] = etl_name
+        return self._client.get_full_url(self._object_path, params)
+
     def put_content(self, content: bytes) -> Header:
         """
         Puts bytes as an object to a bucket in AIS storage.
@@ -226,6 +258,6 @@ class Object:
         """
         self._client.request(
             HTTP_METHOD_DELETE,
-            path=f"{URL_PATH_OBJECTS}/{ self._bck_name }/{ self.name }",
+            path=self._object_path,
             params=self._qparams,
         )
