@@ -7,6 +7,7 @@ import base64
 
 from typing import Any, Mapping, List, Optional, Dict
 
+import msgspec
 from pydantic import BaseModel, validator
 
 from aistore.sdk.namespace import Namespace
@@ -63,20 +64,53 @@ class Smap(BaseModel):
     creation_time: str = ""
 
 
-class BucketEntry(BaseModel):
+class BucketEntry(msgspec.Struct):
     """
     Represents a single entry in a bucket -- an object
+    See cmn/objlist.go/LsoEntry
     """
 
-    name: str
-    size: int = 0
-    checksum: str = ""
-    atime: str = ""
-    version: str = ""
-    target_url: str = ""
-    copies: int = 0
-    flags: int = 0
-    object: "Object" = None
+    n: str
+    cs: str = ""
+    a: str = ""
+    v: str = ""
+    t: str = ""
+    s: int = 0
+    c: int = 0
+    f: int = 0
+    object: Any = None
+
+    @property
+    def name(self):
+        return self.n
+
+    @property
+    def checksum(self):
+        return self.cs
+
+    @property
+    def atime(self):
+        return self.a
+
+    @property
+    def version(self):
+        return self.v
+
+    @property
+    def location(self):
+        return self.t
+
+    @property
+    def size(self):
+        return self.s
+
+    @property
+    def copies(self):
+        return self.c
+
+    @property
+    def flags(self):
+        return self.f
 
     def is_cached(self):
         return (self.flags & (1 << 6)) != 0
@@ -85,26 +119,38 @@ class BucketEntry(BaseModel):
         return (self.flags & ((1 << 5) - 1)) == 0
 
 
-class BucketList(BaseModel):
+class BucketList(msgspec.Struct):
     """
     Represents the response when getting a list of bucket items, containing a list of BucketEntry objects
     """
 
-    uuid: str
-    entries: Optional[List[BucketEntry]] = []
-    continuation_token: str
-    flags: int
+    UUID: str
+    ContinuationToken: str
+    Flags: int
+    Entries: List[BucketEntry] = None
+
+    @property
+    def uuid(self):
+        return self.UUID
+
+    @property
+    def continuation_token(self):
+        return self.ContinuationToken
+
+    @property
+    def flags(self):
+        return self.Flags
+
+    @property
+    def entries(self):
+        return [] if self.Entries is None else self.Entries
 
     def get_entries(self):
+        """
+        Deprecated -- use entries property
+        """
         return self.entries
 
-    # pylint: disable=no-self-argument
-    @validator("entries")
-    def set_entries(cls, entries):
-        if entries is None:
-            entries = []
-        return entries
-
 
 class BucketModel(BaseModel):
     """
@@ -122,6 +168,17 @@ class BucketModel(BaseModel):
         return dict_rep
 
 
+class BsummCtrlMsg(BaseModel):
+    """
+    Represents the bucket summary control message
+    """
+
+    uuid: str
+    prefix: str
+    cached: bool
+    present: bool
+
+
 class JobArgs(BaseModel):
     """
     Represents the set of args to pass when making a job-related request
@@ -196,6 +253,7 @@ class ETLDetails(BaseModel):
     dependencies: Optional[str]
     runtime: Optional[str]  # see ext/etl/runtime/all.go
     chunk_size: int = 0
+    argument: str = ""
 
     @validator("code")
     def set_code(cls, code):  # pylint: disable=no-self-argument
@@ -218,6 +276,7 @@ class InitETLArgs(BaseModel):
     etl_name: str
     communication_type: str
     timeout: str
+    arg_type: str = ""
 
 
 class InitSpecETLArgs(InitETLArgs):
@@ -233,6 +292,7 @@ class InitSpecETLArgs(InitETLArgs):
             "timeout": self.timeout,
             "communication": f"{self.communication_type}://",
             "spec": self.spec,
+            "argument": self.arg_type,
         }
 
 
@@ -256,6 +316,7 @@ class InitCodeETLArgs(InitETLArgs):
             "funcs": self.functions,
             "code": self.code,
             "dependencies": self.dependencies,
+            "argument": self.arg_type,
         }
         if self.chunk_size:
             dict_rep["chunk_size"] = self.chunk_size
@@ -328,6 +389,8 @@ class CopyBckMsg(BaseModel):
     prepend: str
     dry_run: bool
     force: bool
+    latest: bool
+    sync: bool
 
     def as_dict(self):
         return {
@@ -335,6 +398,8 @@ class CopyBckMsg(BaseModel):
             "prepend": self.prepend,
             "dry_run": self.dry_run,
             "force": self.force,
+            "latest-ver": self.latest,
+            "synchronize": self.sync,
         }
 
 
@@ -398,6 +463,22 @@ class TCBckMsg(BaseModel):
         return dict_rep
 
 
+class PrefetchMsg(BaseModel):
+    """
+    API message structure for prefetching objects from remote buckets.
+    """
+
+    object_selection: dict
+    continue_on_err: bool
+    latest: bool
+
+    def as_dict(self):
+        dict_rep = self.object_selection
+        dict_rep["coer"] = self.continue_on_err
+        dict_rep["latest-ver"] = self.latest
+        return dict_rep
+
+
 class TCMultiObj(BaseModel):
     """
     API message structure for transforming or copying multiple objects between buckets
